<?php

namespace Drupal\pagedesigner;

use Drupal\Core\Entity\ContentEntityBase;
use Drupal\Core\Language\LanguageManager;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Service\ElementHandler;

/**
 *
 */
class PagedesignerService {
  /**
   * The output of the last operation.
   *
   * @var mixed
   */
  protected $_output = NULL;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManager
   */
  protected $languageManager = NULL;

  /**
   * The element handler.
   *
   * @var \Drupal\pagedesigner\Service\ElementHandler
   */
  protected $elementHandler = NULL;

  /**
   * Create a new instance.
   *
   * @param \Drupal\Core\Session\AccountInterface $user
   *   The user for which to process. If empty, the current user is used.
   * @param string $langCode
   *   The language to process in. If empty, the current language is used.
   */
  public function __construct(LanguageManager $language_manager, ElementHandler $element_handler) {
    $this->languageManager = $language_manager;
    $this->elementHandler = $element_handler;
  }

  /**
   * Returns the output of the last operation.
   *
   * @return mixed
   *   The output of the last operation.
   */
  public function getOutput() {
    return $this->_output;
  }

  /**
   * Get all pagedesigner fields of the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to get the fields from.
   *
   * @return array
   *   The list of fields, keyed by field name.
   */
  public function getPagedesignerFields(ContentEntityBase $entity) {
    $pagedesignerFields = [];
    $fields = $entity->getFields(TRUE);
    foreach ($fields as $field) {
      if ($field->getFieldDefinition()->getType() == 'pagedesigner_item') {
        $pagedesignerFields[$field->getName()] = $field;
      }
    }
    return $pagedesignerFields;
  }

  /**
   * Adds the container value to the given entity.
   *
   * If no container exists, a container will be created.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity to add the containers to.
   */
  public function addContainer(ContentEntityBase &$entity) {
    $entityManager = \Drupal::service('entity_field.manager');
    $fields = $entityManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($fields as $field) {
      if ($field->getType() == 'pagedesigner_item') {
        if ($entity->get($field->getName())->isEmpty()) {
          $container = $this->getContainer($entity, $field->getName());
          $entity->get($field->getName())->entity = $container;
        }
        elseif (!$entity->isDefaultTranslation()) {
          $container = $entity->get($field->getName())->entity;
          $language = $entity->language()->getId();
          if (!$container->hasTranslation($language)) {
            $container->addTranslation($language);
            $container->save();
          }
        }
      }
    }
  }

  /**
   * Adds the container value to the given entity.
   * If no container exists, a container will be created.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *
   * @return void
   */
  public function saveContainer(ContentEntityBase &$entity) {
    $entityManager = \Drupal::service('entity_field.manager');
    $fields = $entityManager->getFieldDefinitions($entity->getEntityTypeId(), $entity->bundle());
    foreach ($fields as $field) {
      if ($field->getType() == 'pagedesigner_item') {
        if (!$entity->get($field->getName())->isEmpty()) {
          $container = $entity->get($field->getName())->entity;
          $container->entity = $entity;
          $container->save();
        }
      }
    }

  }

  /**
   * Loads or creates the container for the given entity in the language of the entity.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *   The entity being rendered.
   *
   * @return \Drupal\pagedesigner\Entity\Element The container.
   */
  public function getContainer(ContentEntityBase $entity, $fieldname = 'field_pagedesigner_content') {
    $container = $entity->get($fieldname)->entity;
    $language = $entity->language()->getId();
    if ($container == NULL) {
      $container = $this->createContainer($language);
    }
    else {
      $container = $this->getLanguage($container, $language);
    }
    return $container;
  }

  /**
   * Create a new container.
   *
   * @return void
   */
  public function createContainer($language) {
    $container = Element::create(['type' => 'container', 'name' => 'container']);
    $container->langcode->value = $language;
    $container->save();
    return $container;
  }

  /**
   * Undocumented function.
   *
   * @param \Drupal\pagedesigner\Entity\Element $container
   * @param string $language
   *
   * @return void
   */
  public function getLanguage(Element $container, $language) {
    if (!$container->hasTranslation($language)) {
      $container->addTranslation($language);
      $container->save();
    }
    return $container->getTranslation($language);
  }

  /**
   * Get the handler.
   *
   * @param \Drupal\Core\Entity\ContentEntityBase $entity
   *
   * @return Drupal\pagedesigner\Plugin\pagedesigner\HandlerPluginInterface
   */
  protected function getHandler() {
    return $this->elementHandler;
  }

}
